import configparser
import logging
from .const import Const


class ModelConfigBase:
    # DEFAULT section
    CLOUD_CONFIG = "cloud_config"
    SEND_CLOUD = "send_cloud"
    CACHE = "cache"
    SEND_INTERVAL = "send_interval"
    DATA_SEND_ONESHOT = "data_send_oneshot"
    WAIT_CONTAINER_STOP = "wait_container_stop"

    # LOG section
    LOG = "LOG"
    FILE = "file"
    STREAM = "stream"

    # DI section
    DI1 = "DI1"
    DI2 = "DI2"
    TYPE = "type"
    TYPE_IS_POLLING = "type_isPolling"
    INTERVAL = "interval"
    EDGE_TYPE = "edge_type"
    START = "start"

    # DO section
    DO1 = "DO1"
    DO2 = "DO2"
    OUTPUT_STATE = "output_state"
    OUTPUT_STATE_IS_HIGH = "output_state_is_High"
    OUTPUT_TIME = "output_time"
    OUTPUT_DELAY_TIME = "output_delay_time"

    # RS485 section
    RS485_DATA1 = "RS485_Data1"
    RS485_DATA2 = "RS485_Data2"
    RS485_DATA3 = "RS485_Data3"
    RS485_DATA4 = "RS485_Data4"
    METHOD = "method"
    BAUDRATE = "baudrate"
    DATA_SIZE = "data_size"
    PARITY = "parity"
    STOP = "stop"
    DEVICE_ID = "device_id"
    FUNC_CODE = "func_code"
    REGISTER_ADDR = "register_addr"
    REGISTER_COUNT = "register_count"
    ENDIAN = "endian"
    ENDIAN_IS_BIG = "endian_is_big"
    # INTERVAL = "interval"
    DATA_OFFSET = "data_offset"
    DATA_MULTIPLY = "data_multiply"
    DATA_DIVIDER = "data_devider"

    # CPU_temp section
    CPU_TEMP = "CPU_temp"
    POLLING_INTERVAL = "polling_interval"
    # SEND_CLOUD = "send_cloud"

    MAX_INTERVAL = 3600

    def __init__(self):
        self._configs = {}
        self._logging = logging.getLogger(Const.LOGGER_NAME)

    def load(self, config_file_path):
        config = configparser.ConfigParser()
        config.read(config_file_path)

        if not self.parse_default_config(config):
            self._logging.error("config file not found [DEFAULT] section.")
            return False

        self.parse_log_config(config)
        try:
            self.parse_di_config(config)
        except Exception as e:
            self._logging.error("[DI] section is invalid.")
            self._logging.error(e.args)
        try:
            self.parse_do_config(config)
        except Exception as e:
            self._logging.error("[DO] section is invalid.")
            self._logging.error(e.args)
        try:
            self.parse_rs485_config(config)
        except Exception as e:
            self._logging.error("[RS485] section is invalid.")
            self._logging.error(e.args)
        self.parse_cpu_temp_config(config)
        return True

    def check_max_interval(self, val):
        try:
            if val == "":
                return 1

            if isinstance(val, str):
                val = int(val)
            if val > self.MAX_INTERVAL:
                val = self.MAX_INTERVAL
                msg = print(
                    "max interval set {}[s].".format(self.MAX_INTERVAL))
                self._logging.info(msg)
            return val
        except Exception as e:
            self._logging.error("max interval check error.")
            self._logging.error(e.args)
            return 1

    def parse_default_config(self, config):
        default_dict = {}
        section = "DEFAULT"
        try:
            cloud_config = config[section].getboolean(
                ModelConfigBase.CLOUD_CONFIG)
            default_dict[ModelConfigBase.CLOUD_CONFIG] = cloud_config

            send_cloud = config[section].getboolean(
                ModelConfigBase.SEND_CLOUD)
            default_dict[ModelConfigBase.SEND_CLOUD] = send_cloud

            cache = config[section].getboolean(
                ModelConfigBase.CACHE)
            default_dict[ModelConfigBase.CACHE] = cache

            send_interval = self.check_max_interval(
                config[section].getint(ModelConfigBase.SEND_INTERVAL, 10))
            if send_interval < 1:
                send_interval = 1
            elif send_interval > 10:
                send_interval = 10
            default_dict[ModelConfigBase.SEND_INTERVAL] = send_interval

            oneshot = config[section].getboolean(
                ModelConfigBase.DATA_SEND_ONESHOT, False)
            default_dict[ModelConfigBase.DATA_SEND_ONESHOT] = oneshot

            stop = config[section].getint(
                ModelConfigBase.WAIT_CONTAINER_STOP, 0)
            if stop < 0:
                stop = 0
            elif stop > 60:
                stop = 60
            default_dict[ModelConfigBase.WAIT_CONTAINER_STOP] = stop

            self._configs[section] = default_dict
            return True
        except Exception as e:
            self._logging.error(
                "[DEFAULT] section need [cloud_config], [send_cloud], \
                [cache], [send_interval], [data_send_oneshot],\
                [wait_container_stop]")
            self._logging.error(e.args)
        return False

    def parse_log_config(self, config):
        log_dict = {}
        section = ModelConfigBase.LOG
        file = config[section].getboolean(ModelConfigBase.FILE, True)
        log_dict[ModelConfigBase.FILE] = file
        stream = config[section].getboolean(ModelConfigBase.STREAM, False)
        log_dict[ModelConfigBase.STREAM] = stream
        self._configs[section] = log_dict

    # CPU temp
    def parse_cpu_temp_config(self, config):
        cpu_temp_dict = {}
        section = ModelConfigBase.CPU_TEMP
        interval = self.check_max_interval(
            config[section].getint(ModelConfigBase.POLLING_INTERVAL, 1))
        cpu_temp_dict[ModelConfigBase.POLLING_INTERVAL] = interval
        send_cloud = config[section].getboolean(
            ModelConfigBase.SEND_CLOUD, False)
        cpu_temp_dict[ModelConfigBase.SEND_CLOUD] = send_cloud

        self._configs[section] = cpu_temp_dict

    # DI
    def parse_di_config_dict(self, section, config):
        di_dict = {}
        type_isPolling = True

        di_dict[ModelConfigBase.START] = True

        if config[section][ModelConfigBase.TYPE] == "edge":
            type_isPolling = False
        elif config[section][ModelConfigBase.TYPE] == "" or \
                config[section][ModelConfigBase.TYPE] == "none":
            di_dict[ModelConfigBase.START] = False

        type = config[section].get(ModelConfigBase.TYPE, "polling")
        di_dict[ModelConfigBase.TYPE] = type
        di_dict[ModelConfigBase.TYPE_IS_POLLING] = type_isPolling
        interval = self.check_max_interval(
            config[section].get(ModelConfigBase.INTERVAL, 1))
        di_dict[ModelConfigBase.INTERVAL] = interval
        # option
        edge_type = config[section].get(ModelConfigBase.EDGE_TYPE, "")
        di_dict[ModelConfigBase.EDGE_TYPE] = edge_type
        return di_dict

    def parse_di_config(self, config):
        section = ModelConfigBase.DI1
        if config.has_section(section):
            self._configs[section] = self.parse_di_config_dict(section, config)

        section = ModelConfigBase.DI2
        if config.has_section(section):
            self._configs[section] = self.parse_di_config_dict(section, config)

    # DO
    def parse_do_config_dict(self, section, config):
        do_dict = {}

        output_state_isHigh = True
        if config[section][ModelConfigBase.OUTPUT_STATE] == "low":
            output_state_isHigh = False

        do_dict[ModelConfigBase.START] = True
        state = config[section][ModelConfigBase.OUTPUT_STATE]
        do_dict[ModelConfigBase.OUTPUT_STATE] = state
        do_dict[ModelConfigBase.OUTPUT_STATE_IS_HIGH] = output_state_isHigh
        output_time = config[section].getint(ModelConfigBase.OUTPUT_TIME, 1)
        do_dict[ModelConfigBase.OUTPUT_TIME] = output_time
        delay_time = config[section].getint(
            ModelConfigBase.OUTPUT_DELAY_TIME, 0)
        do_dict[ModelConfigBase.OUTPUT_DELAY_TIME] = delay_time

        return do_dict

    def parse_do_config(self, config):
        section = ModelConfigBase.DO1
        if config.has_section(section):
            self._configs[section] = self.parse_do_config_dict(section, config)
        section = ModelConfigBase.DO2
        if config.has_section(section):
            self._configs[section] = self.parse_do_config_dict(section, config)

    # RS485
    def parse_rs485_config_dict(self, section, config):
        rs485_dict = {}
        rs485_dict[ModelConfigBase.START] = False
        rs485_dict[ModelConfigBase.INTERVAL] = 1
        endian_is_big = True

        if config[section][ModelConfigBase.METHOD] != "rtu":
            self._logging.error("RS485 method parameter is not Modbus rtu.")
            return rs485_dict

        if not config[section][ModelConfigBase.BAUDRATE]:
            self._logging.error("RS485 baudrate parameter is empty.")
            return rs485_dict

        if not config[section][ModelConfigBase.PARITY]:
            self._logging.error("RS485 parity parameter is empty.")
            return rs485_dict

        if not config[section][ModelConfigBase.ENDIAN]:
            self._logging.error("RS485 endian parameter is empty.")
            return rs485_dict
        elif config[section][ModelConfigBase.ENDIAN] == "little":
            endian_is_big = False

        rs485_dict[ModelConfigBase.START] = True
        method = config[section].get(ModelConfigBase.METHOD)
        rs485_dict[ModelConfigBase.METHOD] = method
        baudrate = config[section].getint(ModelConfigBase.BAUDRATE)
        rs485_dict[ModelConfigBase.BAUDRATE] = baudrate
        data_size = config[section].getint(ModelConfigBase.DATA_SIZE, 8)
        rs485_dict[ModelConfigBase.DATA_SIZE] = data_size
        parity = config[section].get(ModelConfigBase.PARITY)
        rs485_dict[ModelConfigBase.PARITY] = parity
        stop = config[section].getint(ModelConfigBase.STOP)
        rs485_dict[ModelConfigBase.STOP] = stop
        device_id = config[section].getint(ModelConfigBase.DEVICE_ID)
        rs485_dict[ModelConfigBase.DEVICE_ID] = device_id
        func_code = config[section].getint(ModelConfigBase.FUNC_CODE)
        rs485_dict[ModelConfigBase.FUNC_CODE] = func_code
        register_addr = config[section].getint(ModelConfigBase.REGISTER_ADDR)
        rs485_dict[ModelConfigBase.REGISTER_ADDR] = register_addr
        register_count = config[section].getint(ModelConfigBase.REGISTER_COUNT)
        rs485_dict[ModelConfigBase.REGISTER_COUNT] = register_count
        endian = config[section].get(ModelConfigBase.ENDIAN)
        rs485_dict[ModelConfigBase.ENDIAN] = endian
        rs485_dict[ModelConfigBase.ENDIAN_IS_BIG] = endian_is_big
        interval = self.check_max_interval(
            config[section].get(ModelConfigBase.INTERVAL, 1))
        rs485_dict[ModelConfigBase.INTERVAL] = interval
        # option
        offset = config[section].getint(ModelConfigBase.DATA_OFFSET, 0)
        rs485_dict[ModelConfigBase.DATA_OFFSET] = offset
        multiply = config[section].getint(ModelConfigBase.DATA_MULTIPLY, 0)
        rs485_dict[ModelConfigBase.DATA_MULTIPLY] = multiply
        divider = config[section].getint(ModelConfigBase.DATA_DIVIDER, 0)
        rs485_dict[ModelConfigBase.DATA_DIVIDER] = divider
        return rs485_dict

    def parse_rs485_config(self, config):
        section = ModelConfigBase.RS485_DATA1
        if config.has_section(section):
            self._configs[section] = self.parse_rs485_config_dict(
                section, config)
        section = ModelConfigBase.RS485_DATA2
        if config.has_section(section):
            self._configs[section] = self.parse_rs485_config_dict(
                section, config)
        section = ModelConfigBase.RS485_DATA3
        if config.has_section(section):
            self._configs[section] = self.parse_rs485_config_dict(
                section, config)
        section = ModelConfigBase.RS485_DATA4
        if config.has_section(section):
            self._configs[section] = self.parse_rs485_config_dict(
                section, config)

    # global
    def get_default_conf(self):
        return self._configs["DEFAULT"]

    def get_cpu_temp_conf(self):
        return self._configs[ModelConfigBase.CPU_TEMP]

    def get_log_conf(self):
        return self._configs[ModelConfigBase.LOG]

    # DI
    def get_di_conf(self, port):
        port_name = ""
        if port == 1:
            port_name = ModelConfigBase.DI1
        elif port == 2:
            port_name = ModelConfigBase.DI2

        if port_name in self._configs.keys():
            return self._configs[port_name]
        else:
            conf = self.create_default_di_conf()
            self._configs[port_name] = conf
            return conf

    def create_default_di_conf(self):
        di_dict = {}
        di_dict[ModelConfigBase.START] = False
        di_dict[ModelConfigBase.TYPE] = "polling"
        di_dict[ModelConfigBase.TYPE_IS_POLLING] = True
        di_dict[ModelConfigBase.INTERVAL] = 1
        di_dict[ModelConfigBase.EDGE_TYPE] = ""
        return di_dict

    # DO
    def get_do_conf(self, port):
        port_name = ""
        if port == 1:
            port_name = ModelConfigBase.DO1
        elif port == 2:
            port_name = ModelConfigBase.DO2

        if port_name in self._configs.keys():
            return self._configs[port_name]
        else:
            conf = self.create_default_do_conf()
            self._configs[port_name] = conf
            return conf

    def create_default_do_conf(self):
        do_dict = {}
        do_dict[ModelConfigBase.START] = False
        do_dict[ModelConfigBase.OUTPUT_STATE] = ""
        do_dict[ModelConfigBase.OUTPUT_STATE_IS_HIGH] = True
        do_dict[ModelConfigBase.OUTPUT_TIME] = 0
        do_dict[ModelConfigBase.OUTPUT_DELAY_TIME] = 0
        return do_dict

    def get_rs485_conf(self, data_num):
        data_name = ""
        if data_num == 1:
            data_name = ModelConfigBase.RS485_DATA1
        elif data_num == 2:
            data_name = ModelConfigBase.RS485_DATA2
        elif data_num == 3:
            data_name = ModelConfigBase.RS485_DATA3
        elif data_num == 4:
            data_name = ModelConfigBase.RS485_DATA4

        if data_name in self._configs.keys():
            return self._configs[data_name]
        else:
            conf = self.create_default_rs485_conf()
            self._configs[data_name] = conf
            return conf

    def create_default_rs485_conf(self):
        rs485_dict = {}
        rs485_dict[ModelConfigBase.START] = False
        rs485_dict[ModelConfigBase.METHOD] = "rtu"
        rs485_dict[ModelConfigBase.BAUDRATE] = 0
        rs485_dict[ModelConfigBase.PARITY] = "none"
        rs485_dict[ModelConfigBase.STOP] = 1
        rs485_dict[ModelConfigBase.DATA_SIZE] = 8
        rs485_dict[ModelConfigBase.DEVICE_ID] = 0
        rs485_dict[ModelConfigBase.FUNC_CODE] = 0
        rs485_dict[ModelConfigBase.REGISTER_ADDR] = 0
        rs485_dict[ModelConfigBase.REGISTER_COUNT] = 1
        rs485_dict[ModelConfigBase.ENDIAN] = "big"
        rs485_dict[ModelConfigBase.ENDIAN_IS_BIG] = True
        rs485_dict[ModelConfigBase.INTERVAL] = 1
        # option
        rs485_dict[ModelConfigBase.DATA_OFFSET] = 0
        rs485_dict[ModelConfigBase.DATA_MULTIPLY] = 0
        rs485_dict[ModelConfigBase.DATA_DIVIDER] = 0
        return rs485_dict

    def send_interval(self):
        conf = self.get_default_conf()
        return conf.get(ModelConfigBase.SEND_INTERVAL)

    # update
    def update_di_conf(self, key, dict):
        old_conf = None
        section = ""
        if key == ModelConfigBase.DI1:
            old_conf = self.get_di_conf(1)
            section = ModelConfigBase.DI1
        elif key == ModelConfigBase.DI2:
            old_conf = self.get_di_conf(2)
            section = ModelConfigBase.DI2
        else:
            self._logging.error("DI config key is invalid.")
            return None

        for key, value in dict.items():
            if key == ModelConfigBase.TYPE:
                old_conf[ModelConfigBase.START] = True
                type_isPolling = True
                if value == "edge":
                    type_isPolling = False
                elif value == "none" or value == "":
                    old_conf[ModelConfigBase.START] = False
                old_conf[ModelConfigBase.TYPE_IS_POLLING] = type_isPolling
            old_conf[key] = value

        self._configs[section] = old_conf
        return section

    def update_do_conf(self, key, dict):
        old_conf = None
        section = ""
        if key == ModelConfigBase.DO1:
            old_conf = self.get_do_conf(1)
            section = ModelConfigBase.DO1
        elif key == ModelConfigBase.DO2:
            old_conf = self.get_do_conf(2)
            section = ModelConfigBase.DO2
        else:
            self._logging.error("DO config key is invalid.")
            return None

        for key, value in dict.items():
            if key == ModelConfigBase.OUTPUT_STATE:
                state_isHigh = True
                if value == "low":
                    state_isHigh = False
                old_conf[ModelConfigBase.OUTPUT_STATE_IS_HIGH] = state_isHigh
            old_conf[key] = value

        self._configs[section] = old_conf
        return section

    def update_rs485_conf(self, key, dict):
        old_conf = None

        section = ""
        if key == ModelConfigBase.RS485_DATA1:
            old_conf = self.get_rs485_conf(1)
            section = ModelConfigBase.RS485_DATA1
        elif key == ModelConfigBase.RS485_DATA2:
            old_conf = self.get_rs485_conf(2)
            section = ModelConfigBase.RS485_DATA2
        elif key == ModelConfigBase.RS485_DATA3:
            old_conf = self.get_rs485_conf(3)
            section = ModelConfigBase.RS485_DATA3
        elif key == ModelConfigBase.RS485_DATA4:
            old_conf = self.get_rs485_conf(4)
            section = ModelConfigBase.RS485_DATA4
        else:
            self._logging.error("RS485 config key is invalid.")
            return None

        for key, value in dict.items():
            if key == ModelConfigBase.ENDIAN:
                endian_is_big = True
                if value == "little":
                    endian_is_big = False
                old_conf[ModelConfigBase.ENDIAN_IS_BIG] = endian_is_big
            old_conf[key] = value

        self._configs[section] = old_conf
        return section

    def update_config(self, data):
        update_list = {}

        for key, value in data.items():
            section = None
            if "_config" not in key:
                continue

            index = key.find('_config')
            key = key[:index]

            if "DI" in key:
                section = self.update_di_conf(key, value)
            elif "DO" in key:
                section = self.update_do_conf(key, value)
            elif "RS485" in key:
                section = self.update_rs485_conf(key, value)

            if section is not None:
                update_list[section] = self._configs[section]

        return update_list

#
# End of File
#
