import logging
from .model_dev_base import ModelDevBase
from .reporters.cpu_temp_reporter import CpuTempReporter
from .reporters.di_reporter import DIReporter
from .io.dio_manager import DIO_Mgr
from .reporters.rs485_reporter import RS485Reporter
from .io.rs485_manager import RS485_Mgr
from .reporters.sw_event_reporter import SwEventReporter
from .sensing_manager import SensingMgr
from .const import Const
from .io.led_device import LedDevice


class ModelDevA6EBase(ModelDevBase):
    def __init__(self, modelConfig, cloud_config, send_cloud):
        super().__init__(modelConfig, send_cloud)
        self._cpu_temp_reporter = None
        self._dio_mgr = None
        self._rs485_mgr = None
        self._sensing_mgr = None
        self._reporters = {}
        self._model_config = modelConfig
        self._cloud_config = cloud_config
        self._logging = logging.getLogger(Const.LOGGER_NAME)

    def on_recv_data(self, data):
        if data is None:
            self._logging.info("data from cloud is empty.")
            return False

        if self._cloud_config:
            self._logging.info("update config from cloud.")
            self.update_reporter(self._model_config, data)
        else:
            self._logging.info("config don't update from cloud.")

        return self.exec_command(self._model_config, data)

    def update_reporter(self, config, data):
        update_list = config.update_config(data)

        if not update_list:
            return

        for section, conf in update_list.items():
            if section == config.DI1 or section == config.DI2:
                reporter = self._reporters[section]
                reporter.set_config(conf.get(config.TYPE_IS_POLLING))
                reporter.set_interval(conf.get(config.INTERVAL))
                if conf.get(config.START):
                    self._logging.info("di start.")
                    reporter.start()
                else:
                    self._logging.info("di stop.")
                    reporter.stop()

            elif section == config.RS485_DATA1 or \
                    section == config.RS485_DATA2 or \
                    section == config.RS485_DATA3 or \
                    section == config.RS485_DATA4:

                reporter = self._reporters[section]
                if section == config.RS485_DATA1:
                    data_num = RS485_Mgr.DATA1
                elif section == config.RS485_DATA2:
                    data_num = RS485_Mgr.DATA2
                elif section == config.RS485_DATA3:
                    data_num = RS485_Mgr.DATA3
                elif section == config.RS485_DATA4:
                    data_num = RS485_Mgr.DATA4

                reporter.set_config(
                    data_num=data_num,
                    baudrate=conf.get(config.BAUDRATE),
                    device_id=conf.get(config.DEVICE_ID),
                    stop_bits=conf.get(config.STOP),
                    parity=conf.get(config.PARITY),
                    reg_addr=conf.get(config.REGISTER_ADDR),
                    func_code=conf.get(config.FUNC_CODE),
                    count=conf.get(config.REGISTER_COUNT),
                    endian_isBig=conf.get(config.ENDIAN),
                    data_offset=conf.get(config.DATA_OFFSET),
                    data_multiply=conf.get(config.DATA_MULTIPLY),
                    data_divider=conf.get(config.DATA_DIVIDER),
                    data_as_float=conf.get(config.DATA_AS_FLOAT)
                    )
                reporter.set_interval(conf.get(config.INTERVAL))
                reporter.start()
            elif section == config.DO1 or section == config.DO2:
                port = 0
                if section == config.DO1:
                    port = DIO_Mgr.DO1_PORT
                elif section == config.DO2:
                    port = DIO_Mgr.DO2_PORT

                self._sensing_mgr.set_do_conf(
                    port=port,
                    output_state_isHigh=conf.get(config.OUTPUT_STATE_IS_HIGH),
                    output_time=conf.get(config.OUTPUT_TIME),
                    output_delay_time=conf.get(config.OUTPUT_DELAY_TIME),
                    )

    def exec_command(self, config, data):
        ret = True
        for section, conf in data.items():
            index = section.find('_command')
            section = section[:index]

            if conf == "":
                continue

            if section == config.DO1 or section == config.DO2:
                port = 0
                if section == config.DO1:
                    port = DIO_Mgr.DO1_PORT
                elif section == config.DO2:
                    port = DIO_Mgr.DO2_PORT

                key = "action"
                if key in conf.keys():
                    self._logging.info(f"do is {key}:{conf[key]}")

                    if conf[key] == "start":
                        self._sensing_mgr.start_do(port)
                    elif conf[key] == 'stop':
                        self._sensing_mgr.stop_do(port)
                    else:
                        ret = False
                        self._logging.error(
                            "DO_command valid value is start or stop.")
                else:
                    ret = False
                    self._logging.error("DO_command don't have [action]")

            elif section == "RS485":
                try:
                    self._rs485_mgr.write_command(
                        method=conf.get(config.METHOD),
                        baudrate=conf.get(config.BAUDRATE),
                        data_size=conf.get(config.DATA_SIZE),
                        stop_bits=conf.get(config.STOP),
                        parity=conf.get(config.PARITY),
                        device_id=conf.get(config.DEVICE_ID),
                        reg_addr=conf.get(config.REGISTER_ADDR),
                        func_code=conf.get(config.FUNC_CODE),
                        write_byte=conf.get("write_byte")
                        )
                except Exception as e:
                    ret = False
                    self._logging.error("RS485_command is invalid parameter")
                    self._logging.error(e.args)
            elif section == "led":
                key = "state"

                if key in conf.keys():
                    self._logging.info(f"led is {key}:{conf[key]}")

                    if conf[key] == "on":
                        with LedDevice.get("app") as led:
                            led.on_or_off(True)
                    elif conf[key] == 'off':
                        with LedDevice.get("app") as led:
                            led.on_or_off(False)
                    else:
                        ret = False
                        self._logging.error(
                            "led_command valid value is on or off.")
                else:
                    ret = False
                    self._logging.error("led_command don't have [state]")
        return ret

    def send_interval(self, config):
        conf = config.get_default_conf()
        return conf.get(config.SEND_INTERVAL)

    def _make_active_reporters(self, config, report_queue, report_repository):
        reporters = []
        swreporter = SwEventReporter(
            report_queue, report_repository, "/dev/input/event1")

        reporters.append(swreporter)
        return reporters

    def _make_reporters(self, config):
        # hide super()'s CpuTempReporter
        reporters = []
        conf = config.get_cpu_temp_conf()

        if conf.get(config.SEND_CLOUD):
            tempReporter = CpuTempReporter(
                start=True,
                interval=conf.get(config.POLLING_INTERVAL),
                is_send=conf.get(config.SEND_CLOUD)
                )
            tempReporter.set_interval(conf.get(config.POLLING_INTERVAL))
            reporters.append(tempReporter)
            self._cpu_temp_reporter = tempReporter

        # DI/DO
        dio_mgr = DIO_Mgr((5, 0), (5, 1), (5, 2), (5, 3))

        # DI reporter
        port = DIO_Mgr.DI1_PORT
        while port <= DIO_Mgr.DI2_PORT:
            conf = config.get_di_conf(port)
            if conf is not None:
                diReporter = DIReporter(
                    dio_mgr=dio_mgr,
                    start=conf.get(config.START),
                    type_isPolling=conf.get(config.TYPE_IS_POLLING),
                    port=port
                    )
                diReporter.set_interval(conf.get(config.INTERVAL))
                reporters.append(diReporter)
            if port == DIO_Mgr.DI1_PORT:
                self._reporters[config.DI1] = diReporter
            elif port == DIO_Mgr.DI2_PORT:
                self._reporters[config.DI2] = diReporter
            port += 1
        self._dio_mgr = dio_mgr

        # RS485 reporter
        rs485_mgr = RS485_Mgr("/dev/ttymxc4")
        data_num = RS485_Mgr.DATA1
        while data_num <= RS485_Mgr.DATA4:
            conf = config.get_rs485_conf(data_num)
            if conf is not None:
                rs485Reporter = RS485Reporter(
                    rs485_mgr=rs485_mgr,
                    start=conf.get(config.START),
                    data_num=data_num,
                    method=conf.get(config.METHOD),
                    baudrate=conf.get(config.BAUDRATE),
                    device_id=conf.get(config.DEVICE_ID),
                    data_size=conf.get(config.DATA_SIZE),
                    stop_bits=conf.get(config.STOP),
                    parity=conf.get(config.PARITY),
                    reg_addr=conf.get(config.REGISTER_ADDR),
                    func_code=conf.get(config.FUNC_CODE),
                    count=conf.get(config.REGISTER_COUNT),
                    endian_isBig=conf.get(config.ENDIAN),
                    data_offset=conf.get(config.DATA_OFFSET),
                    data_multiply=conf.get(config.DATA_MULTIPLY),
                    data_divider=conf.get(config.DATA_DIVIDER)
                    )
                rs485Reporter.set_interval(conf.get(config.INTERVAL))
                reporters.append(rs485Reporter)
                if data_num == RS485_Mgr.DATA1:
                    self._reporters[config.RS485_DATA1] = rs485Reporter
                elif data_num == RS485_Mgr.DATA2:
                    self._reporters[config.RS485_DATA2] = rs485Reporter
                elif data_num == RS485_Mgr.DATA3:
                    self._reporters[config.RS485_DATA3] = rs485Reporter
                elif data_num == RS485_Mgr.DATA4:
                    self._reporters[config.RS485_DATA4] = rs485Reporter
            data_num += 1

        self._rs485_mgr = rs485_mgr
        return reporters

    def _make_sensing_manager(self, config):
        sensing_mgr = SensingMgr(self._dio_mgr)
        # DI edge
        port = DIO_Mgr.DI1_PORT
        while port <= DIO_Mgr.DI2_PORT:
            conf = config.get_di_conf(port)
            if conf is not None:
                if not conf.get(config.TYPE_IS_POLLING):
                    sensing_mgr.create_di_conf(
                        port=port,
                        interval=conf.get(config.INTERVAL),
                        edge_type=conf.get(config.EDGE_TYPE))
            port += 1

        port = DIO_Mgr.DO1_PORT
        while port <= DIO_Mgr.DO2_PORT:
            conf = config.get_do_conf(port)
            if conf is not None:
                sensing_mgr.create_do_conf(
                    port=port,
                    start=conf.get(config.START),
                    output_state_isHigh=conf.get(config.OUTPUT_STATE_IS_HIGH),
                    output_time=conf.get(config.OUTPUT_TIME),
                    output_delay_time=conf.get(config.OUTPUT_DELAY_TIME)
                    )
            port += 1
        self._sensing_mgr = sensing_mgr
        return sensing_mgr

#
# End of File
#
